﻿// *****************************************************************
// *                                                               *
// *       Copyright (c) Fast Corporation, 2010                    *
// *                                                               *
// *   All Rights Reserved.  Unpublished rights  reserved  under   *
// *   the copyright laws of the Japan.                            *
// *                                                               *
// *   The software contained on this media  is  proprietary  to   *
// *   and  embodies  the  confidential  technology  of  Fast      *
// *   Corporation.  Possession, use,  duplication  or             *
// *   dissemination of the software and media is authorized only  *
// *   pursuant to a valid written license from Fast Corporation.  *
// *                                                               *
// *****************************************************************

// -----------------------------------------------------------------
// FVIL Sample Program
// メインダイアログ
//
// 作成者 : N.Moriguchi
//
// 履歴 : 2011/04/05 初版
//
// Color Extraction Sample Program
// -----------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace ColorExtraction_Sample
{
    public partial class Form_ColorExtraction_Sample : Form
    {
        //画像メモリ格納領域（読込み画像）
        private FVIL.Data.CFviImage m_img_src = new FVIL.Data.CFviImage();
        //画像メモリ格納領域（色抽出後画像）
        private FVIL.Data.CFviImage m_img_dst = new FVIL.Data.CFviImage();
        //表示オーバーレイクラス宣言
        private FVIL.GDI.CFviOverlay m_overlay = new FVIL.GDI.CFviOverlay();

        //色票配列
        private fvalgcli.F_COLOR_CHART_PTR m_PTRchartArray;
        //ノルムタイプ
        private fvalgcli.f_colorcheck_norm_type m_NormType;
        //重み係数
        private fvalgcli.DOUBLE_PTR m_Weight;

        public Form_ColorExtraction_Sample()
        {
            InitializeComponent();
        }

        /// <summary>
        /// フォーム表示開始
        /// </summary>
        /// <param name="sender"></param>
        /// 使用許諾を確認します。
        /// <param name="e"></param>
        private void Form_ColorExtraction_Sample_Shown(object sender, EventArgs e)
        {
            //使用許諾の表示
            dialogLicence licence_dialog = new dialogLicence();
            if (licence_dialog.ShowDialog(this) != DialogResult.OK)
            {
                //使用許諾が得られない場合はサンプルプログラムの動作を終了
                this.Close();
                return;
            }

            //画像オブジェクトを画像ビューに設定
            cFviImageView1.Image = m_img_src;
            m_overlay.Image = m_img_src;
        }

        /// <summary>
        /// メインフォーム初期化
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// メインフォームの初期化処理。
        /// </remarks>
        private void Form_ColorExtraction_Sample_Load(object sender, EventArgs e)
        {
            // FVILの初期化
            // アプリケーションを起動した後、 この関数を呼び出してライブラリを初期化してください。 
            //  この時、ライセンスの認証が行われます。試用期間中であっても、この処理は必ず行う必要が有ります。 
            // 初期化を行わなかった場合は、プロテクトされたクラスの実行ができません。
            FVIL._SetUp.InitVisionLibrary();

            //FIEライブラリの初期化
            fvalgcli.api.fnFIE_setup();

            //色票配列確保
            m_PTRchartArray = fvalgcli.F_COLOR_CHART_PTR.alloc(1);

            //パラメータの初期化
            m_NormType = fvalgcli.f_colorcheck_norm_type.F_CC_CHEBYSHEV;
            m_Weight = fvalgcli.DOUBLE_PTR.Zero;

            //ノルム種別コンボボックス初期化
            ComboBoxNormType.Items.Clear();
            ComboBoxNormType.Items.Add("チェビシェフノルム");
            ComboBoxNormType.Items.Add("マンハッタンノルム");
            ComboBoxNormType.Items.Add("ユークリッドノルム");
            ComboBoxNormType.Items.Add("重み付きユークリッドノルム");
            ComboBoxNormType.Items.Add("マハラノビスノルム");
            ComboBoxNormType.Items.Add("分散基準");
            ComboBoxNormType.Items.Add("許容範囲基準");
            ComboBoxNormType.SelectedIndex = 0;

            //重み付きユークリッドノルム以外は重み係数を使用しない
            NumericUpDownWeight1.Enabled = false;
            NumericUpDownWeight2.Enabled = false;
            NumericUpDownWeight3.Enabled = false;

            //分散基準、許容範囲基準以外は許容範囲は先頭しか使用しない
            NumericUpDownTolerance2.Enabled = false;
            NumericUpDownTolerance3.Enabled = false;

        }

        /// <summary>
        /// 終了処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// アプリケーション終了処理。
        /// </remarks>
        private void Form_ColorExtraction_Sample_FormClosed(object sender, FormClosedEventArgs e)
        {
            //確保したメモリを開放する
            m_PTRchartArray.Dispose();
            m_Weight.Dispose();
            fvalgcli.api.fnFIE_teardown();
        }

        /// <summary>
        /// Load File ボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 処理対象画像を読み込みます。
        /// </remarks>
        private void ButtonLoadFile_Click(object sender, EventArgs e)
        {
            OpenFileDialog ofdlg = new OpenFileDialog();
            ofdlg.Filter = "画像ファイル(*.bmp;*.dib;*.png;*.jpg;*.jpeg;*.tif;*.tiff;*.raw;*.pat)|*.bmp;*.dib;*.png;*.jpg;*.jpeg;*.tif;*.tiff;*.raw;*.pat";
            ofdlg.Filter += "|Bitmap ファイル (*.bmp;*.dib)|*.bmp;*.dib";
            ofdlg.Filter += "|Png ファイル (*.png)|*.png";
            ofdlg.Filter += "|Jpeg ファイル (*.jpg;*.jpeg)|*.jpg;*.jpeg";
            ofdlg.Filter += "|Tiff ファイル (*.tif;*.tiff)|*.tif;*.tiff";
            ofdlg.Filter += "|Raw ファイル (*.raw)|*.raw";
            ofdlg.Filter += "|すべてのファイル (*.*)|*.*";

            if (ofdlg.ShowDialog(this) == DialogResult.OK)
            {
                //入力画像ファイルのロード
                try
                {
                    //この関数は、簡易的に画像ファイルのロード行う為のものです。 
                    //この関数で指定できないパラメータを指定する必要があれば、 各画像ファイルアクセスクラスを直接使用してください。 
                    FVIL.File.Function.LoadImageFile(ofdlg.FileName, m_img_src, FVIL.PixelMode.Unpacking);

                    //画像オブジェクトを画像ビューに設定
                    cFviImageView1.Image = m_img_src;

                    //再描画
                    cFviImageView1.FitImageSize();
                    cFviImageView1.Refresh();

                    //読み込むファイル形式が固定の場合は、以下のように記述します。
                    //例1) bmpファイルの場合
                    //Dim file As FVIL.File.CFviImageFileBmp = New FVIL.File.CFviImageFileBmp()
                    //file.Load(ofdlg.FileName, m_image)

                    //例2) jpgファイルの場合
                    //Dim file As FVIL.File.CFviImageFileJpg = New FVIL.File.CFviImageFileJpg()
                    //file.Load(ofdlg.FileName, m_image)
                }
                catch (FVIL.CFviException ex)
                {
                    //エラー処理
                    MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                catch (System.Exception ex)
                {
                    //エラー処理
                    MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                //再描画
                cFviImageView1.Refresh();
            }

        }

        /// <summary>
        /// 色票処理範囲設定チェックボックス
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 色票の処理範囲を設定します。
        /// </remarks>
        private void CheckBoxSetArea_CheckedChanged(object sender, EventArgs e)
        {
            if (CheckBoxSetArea.Checked == true)
            {
                //処理範囲の設定
                cFviImageView1.Image = m_img_src;
                cFviImageView1.SetOverlayActive(FVIL.GDI.CFviDrawProcarea.OverlayID, true);
            }
            else
            {
                cFviImageView1.SetOverlayActive(FVIL.GDI.CFviDrawProcarea.OverlayID, false);
            }
            //再描画
            cFviImageView1.Refresh();

        }

        /// <summary>
        /// 色抽出実行ボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 色抽出の処理を実行します。
        /// </remarks>
        private void ButtonExtraction_Click(object sender, EventArgs e)
        {
            //色票取得
            fn_get_color_char();
            //色票から抽出画像生成
            fnColor_extract();

            CheckBoxSetArea.Checked = false;

        }

        /// <summary>
        /// 色票取得関数
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// 色票を取得します。
        private void fn_get_color_char()
        {
            try
            {
                //ファイルから読み込んだ画像情報取得
                int Child_X = m_img_src.Window.X;
                int Child_Y = m_img_src.Window.Y;
                int Child_width = m_img_src.Window.Width;
                int Child_height = m_img_src.Window.Height;

                //画像から色票取得箇所のチャイルド画像を作成する
                fvalgcli.FHANDLE hChild;
                hChild = fvalgcli.api.fnFIE_img_child_alloc(m_img_src.GetFIE(), Child_X, Child_Y, Child_width, Child_height);

                //色票取得
                fvalgcli.api.fnFIE_colorchart_calc_image(hChild, IntPtr.Zero, m_PTRchartArray);
                //チャイルド画像を開放する
                hChild.Dispose();
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }

        /// <summary>
        /// 抽出画像生成関数
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// 色票から抽出画像を生成します。
        private void fnColor_extract()
        {
            try
            {
                fvalgcli.DOUBLE_PTR tolerance = fvalgcli.DOUBLE_PTR.Zero;
                //ファイルから読み込んだ画像情報取得
                int width = m_img_src.HorzSize;
                int height = m_img_src.VertSize;
                FVIL.ImageType type = m_img_src.ImageType;
                int channel = m_img_src.Channel;

                //取得した画像情報を使用して出力用の画像を初期化
                m_img_dst.SetSize(width, height, type, channel);

                //許容範囲を設定
                tolerance = fvalgcli.DOUBLE_PTR.alloc(3);
                tolerance[0] = (double)NumericUpDownTolerance1.Value;
                tolerance[1] = (double)NumericUpDownTolerance2.Value;
                tolerance[2] = (double)NumericUpDownTolerance3.Value;
                //重み係数を設定
                m_Weight.Dispose();
                if (m_NormType == fvalgcli.f_colorcheck_norm_type.F_CC_WEIGHTED_EUCLIDEAN)
                {
                    m_Weight = fvalgcli.DOUBLE_PTR.alloc(3);
                    m_Weight[0] = (double)NumericUpDownWeight1.Value;
                    m_Weight[1] = (double)NumericUpDownWeight2.Value;
                    m_Weight[2] = (double)NumericUpDownWeight3.Value;
                }
                else
                {
                    //重み付きユークリッドノルム以外は重み係数をNULLにする
                    m_Weight = fvalgcli.DOUBLE_PTR.Zero;
                }
                //色抽出
                fvalgcli.api.fnFIE_color_extraction(m_img_src.GetFIE(), m_img_dst.GetFIE(), m_PTRchartArray, 1, m_NormType, m_Weight, tolerance, true);
                cFviImageView1.SetOverlayActive(FVIL.GDI.CFviDrawProcarea.OverlayID, false);

                cFviImageView1.Image = m_img_dst;
                //再描画
                cFviImageView1.Refresh();
                //確保したメモリを開放する
                tolerance.Dispose();
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }

        /// <summary>
        /// ノルムタイプコンボボックス
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// ノルムタイプを選択します。
        /// </remarks>
        private void ComboBoxNormType_SelectionChangeCommitted(object sender, EventArgs e)
        {
            //ノルムタイプ
            switch (ComboBoxNormType.SelectedIndex)
            {
                case 0:             //チェビシェフノルム
                    m_NormType = fvalgcli.f_colorcheck_norm_type.F_CC_CHEBYSHEV;
                    NumericUpDownTolerance2.Enabled = false;
                    NumericUpDownTolerance3.Enabled = false;
                    NumericUpDownWeight1.Enabled = false;
                    NumericUpDownWeight2.Enabled = false;
                    NumericUpDownWeight3.Enabled = false;
                    break;
                case 1:             //マンハッタンノルム
                    m_NormType = fvalgcli.f_colorcheck_norm_type.F_CC_MANHATTAN;
                    NumericUpDownTolerance2.Enabled = false;
                    NumericUpDownTolerance3.Enabled = false;
                    NumericUpDownWeight1.Enabled = false;
                    NumericUpDownWeight2.Enabled = false;
                    NumericUpDownWeight3.Enabled = false;
                    break;
                case 2:             //ユークリッドノルム
                    m_NormType = fvalgcli.f_colorcheck_norm_type.F_CC_EUCLIDEAN;
                    NumericUpDownTolerance2.Enabled = false;
                    NumericUpDownTolerance3.Enabled = false;
                    NumericUpDownWeight1.Enabled = false;
                    NumericUpDownWeight2.Enabled = false;
                    NumericUpDownWeight3.Enabled = false;
                    break;
                case 3:             //重み付きユークリッドノルム
                    m_NormType = fvalgcli.f_colorcheck_norm_type.F_CC_WEIGHTED_EUCLIDEAN;
                    NumericUpDownTolerance2.Enabled = false;
                    NumericUpDownTolerance3.Enabled = false;
                    NumericUpDownWeight1.Enabled = true;
                    NumericUpDownWeight2.Enabled = true;
                    NumericUpDownWeight3.Enabled = true;
                    break;
                case 4:             //マハラノビスノルム
                    m_NormType = fvalgcli.f_colorcheck_norm_type.F_CC_MAHALANOBIS;
                    NumericUpDownTolerance2.Enabled = false;
                    NumericUpDownTolerance3.Enabled = false;
                    NumericUpDownWeight1.Enabled = false;
                    NumericUpDownWeight2.Enabled = false;
                    NumericUpDownWeight3.Enabled = false;
                    break;
                case 5:             //分散基準
                    m_NormType = fvalgcli.f_colorcheck_norm_type.F_CC_VARIANCE;
                    NumericUpDownTolerance2.Enabled = true;
                    NumericUpDownTolerance3.Enabled = true;
                    NumericUpDownWeight1.Enabled = false;
                    NumericUpDownWeight2.Enabled = false;
                    NumericUpDownWeight3.Enabled = false;
                    break;
                case 6:             //許容範囲基準
                    m_NormType = fvalgcli.f_colorcheck_norm_type.F_CC_TOLERANCE;
                    NumericUpDownTolerance2.Enabled = true;
                    NumericUpDownTolerance3.Enabled = true;
                    NumericUpDownWeight1.Enabled = false;
                    NumericUpDownWeight2.Enabled = false;
                    NumericUpDownWeight3.Enabled = false;
                    break;
            }

        }

        /// <summary>
        /// 色票読込ボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 色票を読込みます。
        /// </remarks>
        private void ButtonLoadParam_Click(object sender, EventArgs e)
        {
            fvalgcli.F_COLOR_CHART chartArray;
            chartArray = fvalgcli.F_COLOR_CHART.init();

            openFileDialog1.FileName = "param.ini";
            openFileDialog1.Filter = "ini ファイル (*.ini)|*.ini";
            openFileDialog1.DefaultExt = "*.ini";
            if ((openFileDialog1.FileNames.Length <= 0) || (openFileDialog1.ShowDialog() == DialogResult.Cancel))
            {
                return;
            }
            System.IO.StreamReader Reader = new System.IO.StreamReader(openFileDialog1.FileName);
            chartArray.color[0] = double.Parse(Reader.ReadLine());
            chartArray.color[1] = double.Parse(Reader.ReadLine());
            chartArray.color[2] = double.Parse(Reader.ReadLine());
            chartArray.flag = (fvalgcli.f_colorcheck_flag)(int.Parse(Reader.ReadLine()));
            chartArray.mat[0] = double.Parse(Reader.ReadLine());
            chartArray.mat[1] = double.Parse(Reader.ReadLine());
            chartArray.mat[2] = double.Parse(Reader.ReadLine());
            chartArray.mat[3] = double.Parse(Reader.ReadLine());
            chartArray.mat[4] = double.Parse(Reader.ReadLine());
            chartArray.mat[5] = double.Parse(Reader.ReadLine());
            m_PTRchartArray[0] = chartArray;
            Reader.Close();

        }

        /// <summary>
        /// 色票保存ボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 色票を保存します。
        /// </remarks>
        private void ButtonSaveParam_Click(object sender, EventArgs e)
        {
            int flg = 0;
            String temp;
            SaveFileDialog ofdlg = new SaveFileDialog();
            ofdlg.FileName = "param.ini";
            ofdlg.Filter = "ini ファイル (*.ini)|*.ini";
            ofdlg.DefaultExt = "*.ini";
            ofdlg.OverwritePrompt = true;
            if ((ofdlg.FileNames.Length <= 0) || (ofdlg.ShowDialog() == DialogResult.Cancel))
            {
                return;
            }
            System.IO.StreamWriter Writer = new System.IO.StreamWriter(ofdlg.FileName);
            Writer.WriteLine(m_PTRchartArray[0].color[0]);
            Writer.WriteLine(m_PTRchartArray[0].color[1]);
            Writer.WriteLine(m_PTRchartArray[0].color[2]);
            temp = m_PTRchartArray[0].flag.ToString();
            if (temp.IndexOf("F_CC_CALCULATED_VARIANCE") >= 0)
            {
                flg |= 1;
            }
            if (temp.IndexOf("F_CC_VALID_MAHALANOBIS") >= 0)
            {
                flg |= 2;
            }
            Writer.WriteLine(flg.ToString());
            Writer.WriteLine(m_PTRchartArray[0].mat[0]);
            Writer.WriteLine(m_PTRchartArray[0].mat[1]);
            Writer.WriteLine(m_PTRchartArray[0].mat[2]);
            Writer.WriteLine(m_PTRchartArray[0].mat[3]);
            Writer.WriteLine(m_PTRchartArray[0].mat[4]);
            Writer.WriteLine(m_PTRchartArray[0].mat[5]);
            Writer.Close();

        }

    }
}